@@ -1,8 +1,9 @@
 #include "sharpprotocol.h"
 
+#include "pirrx51hardware.h"
+
 #include "pirexception.h"
 #include <string>
-//#include <iostream>
 
 // Some global communications stuff:
 #include <QMutex>
